# Copyright (c) 2019 Analog Devices, Inc.  All rights reserved.

# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#   - Redistributions of source code must retain the above copyright notice, this 
#     list of conditions and the following disclaimer.
#   - Redistributions in binary form must reproduce the above copyright notice, 
#     this list of conditions and the following disclaimer in the documentation 
#     and/or other materials provided with the distribution.  
#   - Modified versions of the software must be conspicuously marked as such.
#   - This software is licensed solely and exclusively for use with 
#     processors/products manufactured by or for Analog Devices, Inc.
#   - This software may not be combined or merged with other code in any manner 
#     that would cause the software to become subject to terms and conditions 
#     which differ from those listed here.
#   - Neither the name of Analog Devices, Inc. nor the names of its contributors 
#     may be used to endorse or promote products derived from this software 
#     without specific prior written permission.
#   - The use of this software may or may not infringe the patent rights of one 
#     or more patent holders.  This license does not release you from the 
#     requirement that you obtain separate licenses from these patent holders 
#     to use this software.

# THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES, INC. AND CONTRIBUTORS "AS IS" AND 
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
# NON-INFRINGEMENT, TITLE, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
# ARE DISCLAIMED. 
# 
# IN NO EVENT SHALL ANALOG DEVICES, INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE OR CONSEQUENTIAL DAMAGES 
# (INCLUDING, BUT NOT LIMITED TO, DAMAGES ARISING OUT OF CLAIMS OF INTELLECTUAL 
# PROPERTY RIGHTS INFRINGEMENT; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON 
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

__maxvolt = 0
__minvolt = 0
__codeStep = 0
__slopeOfMaxMin = 0

def compute_inl(voltArray, codeStep):
    """Computes the INL on a list of voltages.

    Parameters
    ----------
    voltArray : list[float]
        List of floating point numbers as voltages.
    codeStep : int
        Distance between codes provided while capturing the voltage values.
    """

    # Note down min and max values and compute the slope of straight line passing through them
    global __codeStep, __maxvolt, __minvolt, __slopeOfMaxMin
    __maxvolt = max(voltArray)
    __minvolt = min(voltArray)
    __codeStep = codeStep
    maxIndex = voltArray.index(__maxvolt)
    __slopeOfMaxMin = (__maxvolt - __minvolt) / maxIndex
    inlArray = []
    for x in voltArray:
        inlArray.append(__inl_single(x, voltArray.index(x)))
    return inlArray

def __inl_single(volt, voltIndex):
    # INL is computed as the difference between the actual value and the point 
    # on the straight line joining min and max
    straightLinePoint = voltIndex * __slopeOfMaxMin + __minvolt
    # INL is then multiplied by inverse slope
    inl = (volt - straightLinePoint) * (1/__slopeOfMaxMin)
    # INL is multiplied by the steps of code jumps
    inl *= __codeStep
    return inl

def compute_dnl(voltArray, codeStep, resolution, vspan):
    """Computes the DNL on a list of voltages. The resolution is needed in this case

    Parameters
    ----------
    voltArray : list[float]
        List of floating point numbers as voltages.
    codeStep : int
        Distance between codes provided while capturing the voltage values.
    resolution : int
        Resolution of the DAC
    """

    # Note down min and max values and compute the slope of straight line passing through them
    global __codeStep, __maxvolt, __minvolt, __slopeOfMaxMin
    __maxvolt = max(voltArray)
    __minvolt = min(voltArray)
    __codeStep = codeStep
    dnl_array = []
    lsb = vspan / (2 ** resolution)
    prev_volt = voltArray[0]
    for x in voltArray[1:]:
        dnl_array.append(__dnl_single(prev_volt, x, lsb))
        prev_volt = x
    return dnl_array

def __dnl_single(volt_n, volt_n_plus_1, lsb):
    # DNL is calculated as the deviation in output jump between two codes from 1 LSB.
    return (((volt_n_plus_1 - volt_n) / lsb) / __codeStep) - 1